#!/usr/bin/env python
#
# Duo, A crazy eights game.
# Copyright (C) 2009 The Duo developers.
#
# This file is part of Duo.
#
# Duo is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
[TODO: write this]
"""

import getopt
import logging
import sys

# Install internationalization, once for all (defines _() ).
import gettext
gettext.install('duo', 'locale')


import duo

from duo import ui


# Textual data
NOTICE = 'Duo version ' + duo.__version__ + """
Copyright (C) 2004,2008 The Duo developpers.
Duo comes with ABSOLUTELY NO WARRANTY; for details type `duo -w'.
This is free software, and you are welcome to redistribute it under
certain conditions; see the COPYING file.
"""

WARRANTY = """  NO WARRANTY

  11. BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
mERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
REPAIR OR CORRECTION.

  12. IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,
INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING
OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED
TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY
YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER
PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES.
"""


# Generic command line options.
# Game options are available in ui package.
OPT_UI = 'ui'
OPT_HELP = 'help'
OPT_VERBOSE = 'verbose'
OPT_QUIET = 'quiet'
OPT_WARRANTY = 'warranty'


def print_usage():
    """Prints a short notice about Duo usage.
    """
    print NOTICE
    print 'Usage:\tduo [-q] [-v] [--players nb] [--cards nb] [--name nm]'
    print '\tduo -h|--help | -w|--warranty'


def print_help():
    """Prints a detailed notice about Duo usage.
    """
    print_usage()
    print ''
    print '  -h, --' + OPT_HELP + '\t\tthis screen.'
    print '  -u, --' + OPT_UI + '\t\tthe user interface selection.'
    print '  -v, --' + OPT_VERBOSE + '\t\tincrease output on console'
    print '  -q, --' + OPT_QUIET + '\t\tlimits console output to warnings and errors'
    print '  -w, --' + OPT_WARRANTY + '\tdisplay the licence warranties'
    print '\nGame options:'
    print '      --' + ui.OPT_GAME_PLAYER_NAME + '\t\tthe name of the player'
    print '      --' + ui.OPT_GAME_TRANSPORT + '\tthe game transport layer to use'
    print '      --' + ui.OPT_GAME_NB_PLAYERS + '\t\tthe number of players'
    print '      --' + ui.OPT_GAME_NB_CARDS + '\t\tthe number of cards'
    print '      --' + ui.OPT_GAME_RULESET + '\t\tthe ruleset to use'
    print '      --' + ui.OPT_GAME_START + '\t\timmediately start to play'


def print_gpl_warranty():
    """Prints the warranty section of the GPL license.
    """
    print WARRANTY


def read_command_line():
    """Analyse command line.
    """
    # In the future, packages should be called to complete the list of
    # options they propose.
    OPT_LIST = 'u:hvqcw'
    LONG_OPT_LIST = [OPT_UI + '=',
                     OPT_HELP,
                     OPT_VERBOSE, OPT_QUIET,
                     OPT_WARRANTY,

                     ui.OPT_GAME_PLAYER_NAME + '=',
                     ui.OPT_GAME_TRANSPORT + '=',
                     ui.OPT_GAME_NB_PLAYERS + '=',
                     ui.OPT_GAME_NB_CARDS + '=',
                     ui.OPT_GAME_RULESET + '=',
                     ui.OPT_GAME_START]

    # Default options
    options = {
        OPT_UI: 'tk',
        OPT_VERBOSE: logging.WARNING,
    }

    try:
        opt_list, args = getopt.getopt(
            sys.argv[1:], OPT_LIST, LONG_OPT_LIST)
        if args:
            print_usage()
            print 'Unknown argument:', args
            sys.exit()

    except getopt.GetoptError, err:
        print_usage()
        print err
        sys.exit()

    # In depth analyse
    for opt_name, opt_arg in opt_list:
        if opt_name in ('-h', '--' + OPT_HELP):
            print_help()
            sys.exit()

        if opt_name in ('-w', '--' + OPT_WARRANTY):
            print_gpl_warranty()
            sys.exit()

        if opt_name in ('-u', '--' + OPT_UI):
            options[OPT_UI] = opt_arg

        if opt_name == ('--' + ui.OPT_GAME_PLAYER_NAME):
            options[ui.OPT_GAME_PLAYER_NAME] = opt_arg

        if opt_name == ('--' + ui.OPT_GAME_TRANSPORT):
            options[ui.OPT_GAME_TRANSPORT] = opt_arg

        if opt_name == ('--' + ui.OPT_GAME_NB_PLAYERS):
            options[ui.OPT_GAME_NB_PLAYERS] = int(opt_arg)

        if opt_name == ('--' + ui.OPT_GAME_NB_CARDS):
            options[ui.OPT_GAME_NB_CARDS] = int(opt_arg)

        if opt_name == ('--' + ui.OPT_GAME_RULESET):
            options[ui.OPT_GAME_RULESET] = opt_arg

        if opt_name == ('--' + ui.OPT_GAME_START):
            options[ui.OPT_GAME_START] = True

        if opt_name in ('-v', '--' + OPT_VERBOSE):
            if options[OPT_VERBOSE] != 0:
                options[OPT_VERBOSE] = logging.DEBUG
            else:
                print 'verbose option conflicts with quiet.'
                sys.exit()

        if opt_name in ('-q', '--' + OPT_QUIET):
            if options[OPT_VERBOSE] != logging.DEBUG:
                options[OPT_VERBOSE] = 0
            else:
                print 'verbose option conflicts with quiet.'
                sys.exit()

    # Configure all future instances of Logger
    handler = logging.FileHandler('duo.log', 'w')
    formatter = logging.Formatter(
        '%(asctime)s %(name)-25s:%(lineno) 4d\t%(levelname)s\t%(message)s')

    handler.setFormatter(formatter)

    logger = logging.getLogger('duo')
    logger.addHandler(handler)
    logger.setLevel(options[OPT_VERBOSE])

    # Return behaviour
    return options




# Main
try:
    options = read_command_line()

    # Set default language.
    duo.switch_to_language(duo.get_default_language())

    ui = ui.create_ui(options[OPT_UI], options)
    ui.launch()

except KeyboardInterrupt, e:
    print e
except Exception, e:
    print e.message.encode('utf-8')
